/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include "gtk-ppp.h"


void on_conn_disconnect (GtkWidget *widget, gpointer data)
{
	/* try to kill wvdial process */
	if (wvdial.pid != 0)
		if (kill (wvdial.pid, 2) < 0)
			g_print ("GTK PPP: Unable to KILL wvdial process!\n");

	if (gtk_ppp.connected.update > 0) {
		g_source_remove(gtk_ppp.connected.update);
		gtk_ppp.connected.update = 0;
		gtk_ppp.connected.seconds = 0;
		if (gtk_ppp.connected.primarydns != NULL) {
			g_free(gtk_ppp.connected.primarydns);
			gtk_ppp.connected.primarydns = NULL;
		}
		if (gtk_ppp.connected.secondarydns != NULL) {
			g_free(gtk_ppp.connected.secondarydns);
			gtk_ppp.connected.secondarydns = NULL;
		}
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.dns), _("Unknown"));
		gtk_label_set_markup (GTK_LABEL (gtk_ppp.connected.time), _("<big>Connected: 00:00:00</big>"));
		gtk_window_set_title (GTK_WINDOW (gtk_ppp.connected.window), "00:00:00");
	}
	gtk_widget_hide (gtk_ppp.connected.window);
	gtk_widget_hide (gtk_ppp.details.window);
	gtk_widget_hide (gtk_ppp.log.window);
	gtk_widget_show (gtk_ppp.gtk_ppp.window);
}

void on_conn_details (GtkWidget *widget, gpointer data)
{
	gtk_widget_show (gtk_ppp.details.window);
}

gboolean on_conn_delete_event (GtkWidget *widget, GdkEvent *event, gpointer data)
{
	return TRUE;
}

gboolean timer (gpointer data)
{
	gchar *caption = NULL;
	gchar *caption_markup = NULL;
	guint hours = 0;
	guint minutes = 0;
	guint seconds = 0;

	gtk_ppp.connected.seconds++;

	hours = gtk_ppp.connected.seconds / 3600;
	minutes = (gtk_ppp.connected.seconds - hours * 3600) / 60;
	seconds = gtk_ppp.connected.seconds - hours * 3600 - minutes * 60;

	caption = g_strdup_printf ("%2d:%2d:%2d", hours, minutes, seconds);
	caption = g_strcanon (caption, ":0123456789", '0');

	caption_markup = g_strdup_printf (_("<big>Connected: %s</big>"), caption);

	gtk_label_set_markup (GTK_LABEL (gtk_ppp.connected.time), caption_markup);
	gtk_window_set_title (GTK_WINDOW (gtk_ppp.connected.window), caption);

	g_free (caption);
	g_free (caption_markup);

	/* get details on connection */
	/* transfer details */
	gchar *error = NULL;
	glong packets_in = 0;
	glong packets_out = 0;
	glong bytes_in = 0;
	glong bytes_out = 0;
	gchar *kbytes_in = NULL;
	gchar *kbytes_out = NULL;
	const gchar *iface;
	gchar *in = NULL;
	gchar *out = NULL;
	gchar *in_packets = NULL;
	gchar *out_packets = NULL;

	iface = gtk_label_get_text (GTK_LABEL (gtk_ppp.details.interface));
	error = get_transfer_details (iface, &packets_in, &packets_out, &bytes_in, &bytes_out);

	if (bytes_in > gtk_ppp.connected.old_bytes_in && bytes_out > gtk_ppp.connected.old_bytes_out) {
		glong tmp_in = bytes_in - gtk_ppp.connected.old_bytes_in;
		glong tmp_out = bytes_out - gtk_ppp.connected.old_bytes_out;

		tmp_in *= 10;
		tmp_out *= 10;
		tmp_in = tmp_in / 1024;
		tmp_out = tmp_out / 1024;
		kbytes_in = g_strdup_printf ("%ld.%ld KB/s", tmp_in / 10, tmp_in % 10);
		kbytes_out = g_strdup_printf ("%ld.%ld KB/s", tmp_out / 10, tmp_out % 10);

		gtk_ppp.connected.old_bytes_in = bytes_in;
		gtk_ppp.connected.old_bytes_out = bytes_out;

		if (gtk_ppp.dock == TRUE)
			update_notification (0); /* sending/receiving */

		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.status), _("Sending/Receiving"));
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.speed_in), kbytes_in);
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.speed_out), kbytes_out);

		g_free (kbytes_in);
		g_free (kbytes_out);
	} else if (bytes_in > gtk_ppp.connected.old_bytes_in && bytes_out == gtk_ppp.connected.old_bytes_out) {
		glong tmp_in = bytes_in - gtk_ppp.connected.old_bytes_in;

		tmp_in *= 10;
		tmp_in = tmp_in / 1024;
		kbytes_in = g_strdup_printf ("%ld.%ld KB/s", tmp_in / 10, tmp_in % 10);

		gtk_ppp.connected.old_bytes_in = bytes_in;

		if (gtk_ppp.dock == TRUE)
			update_notification (1); /* receiving */

		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.status), _("Receiving"));
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.speed_in), kbytes_in);
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.speed_out), _("0.0 KB/s"));

		g_free (kbytes_in);
	} else if (bytes_in == gtk_ppp.connected.old_bytes_in && bytes_out > gtk_ppp.connected.old_bytes_out) {
		glong tmp_out = bytes_out - gtk_ppp.connected.old_bytes_out;

		tmp_out *= 10;
		tmp_out = tmp_out / 1024;
		kbytes_out = g_strdup_printf ("%ld.%ld KB/s", tmp_out / 10, tmp_out % 10);

		gtk_ppp.connected.old_bytes_out = bytes_out;

		if (gtk_ppp.dock == TRUE)
			update_notification (2); /* sending */

		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.status), _("Sending"));
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.speed_in), _("0.0 KB/s"));
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.speed_out), kbytes_out);

		g_free (kbytes_out);
	} else if (bytes_out == gtk_ppp.connected.old_bytes_out && bytes_in == gtk_ppp.connected.old_bytes_in) {
		gtk_ppp.connected.old_bytes_in = bytes_in;
		gtk_ppp.connected.old_bytes_out = bytes_out;

		if (gtk_ppp.dock == TRUE)
			update_notification (3); /* idle */

		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.status), _("Idle"));
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.speed_in), _("0.0 KB/s"));
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.speed_out), _("0.0 KB/s"));
	}

	bytes_in *= 10;
	bytes_out *= 10;

	if (bytes_in > 1024) { /* KB */
		bytes_in = bytes_in / 1024;
		in = g_strdup_printf ("%ld.%ld KB", bytes_in / 10, bytes_in % 10);
	} else if (bytes_in >= 0) { /* B */
		in = g_strdup_printf ("%ld.%ld B", bytes_in / 10, bytes_in % 10);
	}

	if (bytes_out > 1024) { /* KB */
		bytes_out = bytes_out / 1024;
		out = g_strdup_printf ("%ld.%ld KB", bytes_out / 10, bytes_out % 10);
	} else if (bytes_out >= 0) { /* B */
		out = g_strdup_printf ("%ld.%ld B", bytes_out / 10, bytes_out % 10);
	}

	in_packets = g_strdup_printf ("%ld", packets_in);
	out_packets = g_strdup_printf ("%ld", packets_out);

	if (error == NULL) {
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.received), in);
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.sent), out);
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.received_packets), in_packets);
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.sent_packets), out_packets);
	} else {
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.received), "0");
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.sent), "0");
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.received_packets), "0");
		gtk_label_set_text (GTK_LABEL (gtk_ppp.details.sent_packets), "0");
		g_free (error);
	}

	if (in != NULL)
		g_free (in);

	if (out != NULL)
		g_free (out);

	if (in_packets != NULL)
		g_free (in_packets);

	if (out_packets != NULL)
		g_free (out_packets);
	/* address details */
	gchar *addr = NULL;
	gchar *dest = NULL;

	if (get_addr_details (iface, &addr, &dest)) {

		if (addr)
			gtk_label_set_text (GTK_LABEL (gtk_ppp.details.ip), addr);

		else
			gtk_label_set_text (GTK_LABEL (gtk_ppp.details.ip), _("Unknown"));

		if (dest)
			gtk_label_set_text (GTK_LABEL (gtk_ppp.details.destination), dest);
		else
			gtk_label_set_text (GTK_LABEL (gtk_ppp.details.destination), _("Unknown"));

		if (addr != NULL)
			g_free (addr);

		if (dest != NULL)
			g_free (dest);
	}

	return TRUE;
}
