/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include "gtk-ppp.h"


void init_notification (void)
{
	gchar *mainimg;
	gchar *srimg;
	gchar *simg;
	gchar *rimg;
	gchar *iimg;

	if (gtk_ppp.notification.icon == NULL) {
		gtk_ppp.notification.icon = gtk_status_icon_new();
	}

	g_signal_connect(G_OBJECT (gtk_ppp.notification.icon), "activate",
			G_CALLBACK (on_notification_bpress_event), NULL);

	/* notification icon paths */
	mainimg = g_build_filename (PACKAGE_APPICON_DIR, "gtk_ppp.png", NULL);
	srimg = g_build_filename (PACKAGE_APPICON_DIR, "gtk_ppp_sending_receiving.png", NULL);
	rimg = g_build_filename (PACKAGE_APPICON_DIR, "gtk_ppp_receiving.png", NULL);
	simg = g_build_filename (PACKAGE_APPICON_DIR, "gtk_ppp_sending.png", NULL);
	iimg = g_build_filename (PACKAGE_APPICON_DIR, "gtk_ppp_idle.png", NULL);

	gtk_ppp.notification.mainimg = gdk_pixbuf_new_from_file(mainimg, NULL);
	gtk_ppp.notification.sendingreceivingimg = gdk_pixbuf_new_from_file(srimg, NULL);
	gtk_ppp.notification.receivingimg = gdk_pixbuf_new_from_file(rimg, NULL);
	gtk_ppp.notification.sendingimg = gdk_pixbuf_new_from_file(simg, NULL);
	gtk_ppp.notification.idleimg = gdk_pixbuf_new_from_file(iimg, NULL);

	gtk_status_icon_set_from_pixbuf(gtk_ppp.notification.icon, gtk_ppp.notification.mainimg);
	gtk_status_icon_set_title(gtk_ppp.notification.icon, _("GTK PPP"));

	/* free icon paths */
	g_free (mainimg);
	g_free (srimg);
	g_free (rimg);
	g_free (simg);
	g_free (iimg);

	/* popup menu */
	GtkWidget *item;

	if (gtk_ppp.notification.menu == NULL)
		gtk_ppp.notification.menu = gtk_menu_new ();

	item = gtk_menu_item_new_with_label(_("Connection log"));
	g_signal_connect (G_OBJECT (item), "activate",
			G_CALLBACK (on_notification_menu), "log");
	gtk_menu_shell_append (GTK_MENU_SHELL (gtk_ppp.notification.menu), item);
	gtk_widget_show (item);

	item = gtk_menu_item_new_with_label(_("Details"));
	g_signal_connect (G_OBJECT (item), "activate",
			G_CALLBACK (on_notification_menu), "details");
	gtk_menu_shell_append (GTK_MENU_SHELL (gtk_ppp.notification.menu), item);
	gtk_widget_show (item);

	item = gtk_menu_item_new_with_label(_("Disconnect"));
	g_signal_connect (G_OBJECT (item), "activate",
			G_CALLBACK (on_notification_menu), "disconnect");
	gtk_menu_shell_append (GTK_MENU_SHELL (gtk_ppp.notification.menu), item);
	gtk_widget_show (item);

	item = gtk_separator_menu_item_new ();
	gtk_menu_shell_append (GTK_MENU_SHELL (gtk_ppp.notification.menu), item);
	gtk_widget_show (item);

	/*item = gtk_image_menu_item_new_with_label (_("About"));
	g_signal_connect (G_OBJECT (item ), "activate",
			G_CALLBACK (on_notification_menu), "about");
	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
					gtk_image_new_from_stock (GNOME_STOCK_ABOUT, GTK_ICON_SIZE_MENU));
	gtk_menu_shell_append (GTK_MENU_SHELL (gtk_ppp.notification.menu), item);
	gtk_widget_show (item);*/

	gtk_widget_show_all(gtk_ppp.notification.menu);

	g_signal_connect(G_OBJECT (gtk_ppp.notification.icon), "popup-menu",
			G_CALLBACK (on_notification_menu_event), NULL);
}

void update_notification (guint status)
{
	if (status == 0) {
		/* sending_receiving */
		if (gtk_ppp.notification.sendingreceivingimg != NULL) {
			gtk_status_icon_set_from_pixbuf(gtk_ppp.notification.icon, gtk_ppp.notification.sendingreceivingimg);
		}
	} else if (status == 1) {
		/* receiving */
		if (gtk_ppp.notification.receivingimg != NULL) {
			gtk_status_icon_set_from_pixbuf(gtk_ppp.notification.icon, gtk_ppp.notification.receivingimg);
		}
	} else if (status == 2) {
		/* sending */
		if (gtk_ppp.notification.sendingimg != NULL) {
			gtk_status_icon_set_from_pixbuf(gtk_ppp.notification.icon, gtk_ppp.notification.sendingimg);
		}
	} else if (status == 3) {
		/* idle */
		if (gtk_ppp.notification.idleimg != NULL) {
			gtk_status_icon_set_from_pixbuf(gtk_ppp.notification.icon, gtk_ppp.notification.idleimg);
		}
	}
}

void show_notification (gboolean show)
{
	if (gtk_ppp.notification.icon) {
		gtk_status_icon_set_visible(gtk_ppp.notification.icon, show);
	}
}

void on_notification_bpress_event (GtkStatusIcon *statusicon, gpointer userdata)
{
	if (gtk_ppp.connected.show == TRUE) {
		gtk_widget_hide(gtk_ppp.connected.window);
		gtk_ppp.connected.show = FALSE;
	} else {
		gtk_widget_show (gtk_ppp.connected.window);
		gtk_ppp.connected.show = TRUE;
	}
}

void on_notification_menu_event (GtkStatusIcon *statusicon, guint button, guint activatetime, gpointer userdata)
{
	if (gtk_ppp.notification.menu) {
		gtk_menu_popup(GTK_MENU(gtk_ppp.notification.menu), NULL, NULL, gtk_status_icon_position_menu, statusicon, button, activatetime);
	}
}

void on_notification_menu (GtkWidget *widget, gpointer data)
{
	gchar *tmp = NULL;

	tmp = (gchar *)data;

	if (g_strrstr (data, "details") != NULL)
		gtk_widget_show (gtk_ppp.details.window);

	if (g_strrstr (data, "disconnect") != NULL)
		on_conn_disconnect (NULL, NULL);

	if (g_strrstr (data, "about") != NULL) {
		/*gchar *icon;

		icon = g_build_filename (PACKAGE_APPICON_DIR, "gtk-ppp.png", NULL);

		static const gchar *authors [] = {
			"",
			NULL
		};
		const gchar *documenters [] = { NULL };
		const gchar *translator_credits = _("translator_credits");
		gtk_ppp.about = gnome_about_new (_("GTK PPP"), VERSION,
				"Copyright \xc2\xa9 2004-2019\n",
				_("GTK Dialup Tool"), authors, documenters,
				strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
				gdk_pixbuf_new_from_file (icon, NULL));
		g_free (icon);

		g_signal_connect (G_OBJECT (gtk_ppp.about), "destroy",
				G_CALLBACK (gtk_widget_destroyed), &gtk_ppp.about);

		gtk_widget_show (gtk_ppp.about);*/
	}

	if (g_strrstr (data, "log") != NULL)
		gtk_widget_show (gtk_ppp.log.window);
}

void destroy_notification (void)
{
	gtk_widget_destroy (gtk_ppp.notification.menu);
	g_object_unref (G_OBJECT (gtk_ppp.notification.icon));
	gtk_ppp.notification.icon = NULL;
	gtk_ppp.notification.menu = NULL;

	g_object_unref(gtk_ppp.notification.mainimg);
	gtk_ppp.notification.mainimg = NULL;
	g_object_unref(gtk_ppp.notification.sendingreceivingimg);
	gtk_ppp.notification.sendingreceivingimg = NULL;
	g_object_unref(gtk_ppp.notification.receivingimg);
	gtk_ppp.notification.receivingimg = NULL;
	g_object_unref(gtk_ppp.notification.sendingimg);
	gtk_ppp.notification.sendingimg = NULL;
	g_object_unref(gtk_ppp.notification.idleimg);
	gtk_ppp.notification.idleimg = NULL;
}
